/*
** Module   : BaseEntity.java
** Abstract : server side base class for most widgets
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 NVS 20051026   @23479 Created initial version.
** 002 GES 20060116   @23947 Implemented conversion of 1-based to 0-based positions.
** 003 SIY 20060209   @24432 Added missing method for width setting, other minor changes.
** 004 SIY 20060424   @25680 Changed setParent() signature.
** 005 GES 20060803   @28363 Major cleanup and refactoring.
** 006 NVS 20061204   @31549 SELECTED attribute is server side only from now on.
** 007 NVS 20070426   @33241 SENSITIVE attribute is no longer maintained separately, but is backed
**                           up with ENABLED flag of CHARVA on the client.
** 008 NVS 20080307   @37409 setRow() and setColumn() methods now support dynamic settings that is
**                           required for ROW and COL widget attributes support.
** 009 GES 20080319   @37579 Added NEXT-CHILD and PREV-CHILD support.
** 010 CA  20080925   @39960 Support for dynamic browse title - base field is now protected.
** 011 NVS 20081211   @40872 getParent() method takes into account field group the widget is in.
** 012 GES 20081230   @41001 Return a NullWidget instance instead of null for safety.
** 013 SIY 20101004          Added necessary method stubs.
** 014 SIY 20101005          Generalized setX/setY.
** 015 SVL 20101006          Added FGCOLOR, BGCOLOR attributes.
** 016 SIY 20101007          Use NumberType instead of decimal.
** 017 SIY 20110112          Changes related to new color handling scheme.
** 018 SIY 20111101          Updated imports.
** 019 CA  20130112          getNextSibling and getPrevSibling must return a handle instance.
** 020 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 021 CA  20130323          The parent must be a CommonWidget, not Object.
** 022 OM  20130527          Added int64 support for colors.
** 023 CA  20130927          Fixed getNext/PrevSibling - NullWidget is not needed.
** 024 CA  20131115          Changes related to properly instantiate dynamic resources.
** 025 HC  20131215          Removed PRIVATE-DATA attribute, it is already implemented 
**                           in the base class.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * The server side base class for most widgets. 
 */
public class BaseEntity
extends GenericWidget
{
   /** Reference to our configuration which is pre-cast to the proper type. */
   protected BaseConfig base = null;
   
   /** PARENT attribute */
   private CommonWidget parent = null;
   
   /** Group container for the widget and its siblings. */
   protected Group group = null;
   
   /** PRIVATE-DATA attribute. */
   private String privateData = "";

   /** SELECTED attribute. */
   private logical selected = new logical(false);

   /**
    * Constructor.
    *
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    * @param    base
    *           The container for all state to be transferred to the client.
    */
   public BaseEntity(boolean dynamic, BaseConfig base)
   {
      super(dynamic, base);
      this.base = base;
   }
   
   /**
    * Constructor.
    *
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    * @param    base
    *           The container for all state to be transferred to the client.
    * @param    asLast
    *           Flag indicating if this entity must be inserted as last element in chain
    */
   public BaseEntity(boolean dynamic, BaseConfig base, boolean asLast)
   {
      super(dynamic, base, asLast);
      this.base = base;
   }
   
   /**
    * Gets the COLUMN writable attribute.
    *
    * @return   The 1-based column number at which to start displaying the 
    *           widget.  This is converted from a 0-based column number for
    *           the internal runtime.
    */
   @Override
   public decimal getColumn()
   {
      int col = base.getColumn() + 1;
      return new decimal(col);
   }

   /**
    * Sets the column position for this widget.
    *
    * @param    c
    *           The 1-based row number at which to start displaying the 
    *           widget.  This is converted to a 0-based column number for the
    *           internal runtime.
    */
   @Override
   public void setColumn(NumberType c)
   {
      setColumn(c.intValue());
   }

   /**
    * Sets the column position for this widget.
    *
    * @param    c
    *           The 1-based row number at which to start displaying the 
    *           widget.  This is converted to a 0-based column number for the
    *           internal runtime.
    */
   @Override
   public void setColumn(double c)
   {
      int newCol = (int)c - 1;
      // TO-DO:
      // the column should be checked for validity
      // value < 1 should produce PROGRESS error 4081
      // "**Attribute COL must be greater than or equal to 1 on FRAME f. (4081)"
      // value > screen-width - frame-width should produce PROGRESS error 4054
      // "**Unable to set COL.  FRAME f does not fit in WINDOW widget. (4054)"
      base.setColumn(newCol);

      if (frame != null && !frame.isBatch())
      {
         LogicalTerminal.setLocation(frame, getId(), base.getRow(),
                                                     base.getColumn());
      }

      pushScreenDefinition();
   }

   /**
    * Sets the DCOLOR writable attribute.
    *
    * @param    dcolor
    *           The new value for the DCOLOR attribute.
    */
   @Override
   public void setDcolor(Color dcolor)
   {
      base.setDcolor(dcolor);
      pushScreenDefinition();
   }

   /**
    * Gets the FGCOLOR writable attribute.
    *
    * @return   The current value of the FGCOLOR attribute.
    */
   public integer getFgcolor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Sets the FGCOLOR writable attribute.
    *
    * @param    fgcolor
    *           The new value for the FGCOLOR attribute.
    */
   public void setFgcolor(int64 fgcolor)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Sets the FGCOLOR writable attribute.
    *
    * @param    fgcolor
    *           The new value for the FGCOLOR attribute.
    */
   public void setFgcolor(int fgcolor)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Gets the BGCOLOR writable attribute.
    *
    * @return   The current value of the BGCOLOR attribute.
    */
   public integer getBgcolor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Sets the BGCOLOR writable attribute.
    *
    * @param    bgcolor
    *           The new value for the BGCOLOR attribute.
    */
   public void setBgcolor(int64 bgcolor)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Sets the BGCOLOR writable attribute.
    *
    * @param    bgcolor
    *           The new value for the BGCOLOR attribute.
    */
   public void setBgcolor(int bgcolor)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Gets the HEIGHT-CHARS writable attribute.
    *
    * @return   The current value of the HEIGHT-CHARS attribute.
    */
   @Override
   public decimal getHeightChars()
   {
      return new decimal(base.getHeightChars());
   }

   /**
    * Sets the HEIGHT-CHARS writable attribute.
    *
    * @param    height
    *           The new value for the HEIGHT-CHARS attribute.
    */
   public void setHeightChars(NumberType height)
   {
      setHeightChars(height.intValue());
   }
   
   /**
    * Sets the HEIGHT-CHARS writable attribute.
    *
    * @param    height
    *           The new value for the HEIGHT-CHARS attribute.
    */
   public void setHeightChars(double height)
   {
      base.setHeightChars((int) height);
      pushScreenDefinition();
   }

   /**
    * Gets the WIDTH-CHARS writable attribute.
    *
    * @return   The current value of the WIDTH-CHARS attribute.
    */
   public decimal getWidthChars()
   {
      return new decimal(base.getWidthChars());
   }

   /**
    * Sets the WIDTH-CHARS writable attribute.
    *
    * @param    width
    *           The new value for the WIDTH-CHARS attribute.
    */
   public void setWidthChars(NumberType width)
   {
      setWidthChars(width.intValue());
   }
   
   /**
    * Sets the WIDTH-CHARS writable attribute.
    *
    * @param    width
    *           The new value for the WIDTH-CHARS attribute.
    */
   public void setWidthChars(double width)
   {
      base.setWidthChars((int) width);
      pushScreenDefinition();
   }
   
   /**
    * Gets the PARENT writable attribute.
    * If the parent attribute was set explicitly, it is returned.
    * Otherwise, the field group of this widget is returned as parent.
    * For the regular widgets, it always has a value. It's the default field
    * group. Non-regular widgets override this method to return a different
    * result.  
    *
    * @return   the parent widget for this widget
    */
   @Override
   public CommonWidget getParent()
   {
      // explicitly set parent attribute
      if (parent != null)
      {
         return parent;
      }

      // normal case
      return group;
   }

   /**
    * Sets the PARENT writable attribute.
    *
    * @param    widget
    *           The new value for the PARENT attribute.
    */
   public void setParent(GenericWidget widget)
   {
      this.parent = widget;
      base.setParent(widget.getId());
      pushScreenDefinition();
   }

   /**
    * Establishes a relationship between the container and the elements in
    * a given widget group.
    *
    * @param    group
    *           The container of the widget.
    */
   @Override
   public void setGroup(Group group)
   {
      this.group = group;
   }
   
   /**
    * Obtains the next sibling widget in the current field group.
    *
    * @return   The next sibling or an unknown handle if this is the only widget or if this is the 
    *           last widget.
    */
   @Override
   public handle getNextSibling()
   {
      if (this instanceof HandleChain)
      {
         return super.getNextSibling();
      }
      else
      {
         GenericWidget widget = (group == null) ? null : group.getNextChild(this);
         return new handle(widget);
      }
   }
   
   /**
    * Obtains the previous sibling widget in the current field group.
    *
    * @return   The previous sibling or an unknown handle if this only widget or if this is the 
    *           first widget.
    */
   @Override
   public handle getPrevSibling()
   {
      if (this instanceof HandleChain)
      {
         return super.getPrevSibling();
      }
      else
      {
         GenericWidget widget = (group == null) ? null : group.getPrevChild(this);
         return new handle(widget);
      }
   }

   /**
    * Sets the PFCOLOR writable attribute.
    *
    * @param    pfcolor
    *           The new value for the PFCOLOR attribute.
    */
   public void setPfColor(Color pfcolor)
   {
      base.setPfcolor(pfcolor);
      pushScreenDefinition();
   }

   /**
    * Gets the ROW writable attribute.
    *
    * @return   The 1-based row number at which to start displaying the 
    *           widget.  This is converted from a 0-based row number for the
    *           internal runtime.
    */
   @Override
   public decimal getRow()
   {
      int row = base.getRow() + 1;
      return new decimal(row);
   }

   /**
    * Sets the row position for this widget.
    *
    * @param    r
    *           The 1-based row number at which to start displaying the 
    *           widget.  This is converted to a 0-based row number for the
    *           internal runtime.
    */
   @Override
   public void setRow(NumberType r)
   {
      setRow(r.intValue());
   }

   /**
    * Sets the row position for this widget.
    *
    * @param    r
    *           The 1-based row number at which to start displaying the 
    *           widget.  This is converted to a 0-based row number for the
    *           internal runtime.
    */
   @Override
   public void setRow(double r)
   {
      int newRow = (int)r - 1;

      // TO-DO:
      // the row should be checked for validity
      // value < 1 should produce PROGRESS error 4081
      // "**Attribute ROW must be greater than or equal to 1 on FRAME f. (4081)"
      // value > screen-height - frame-height should produce PROGRESS error 4054
      // "**Unable to set ROW.  FRAME f does not fit in WINDOW widget. (4054)"
      base.setRow(newRow);

      if (frame != null && !frame.isBatch())
      {
         LogicalTerminal.setLocation(frame, getId(), base.getRow(),
                                                     base.getColumn());
      }
   }

   /**
    * Gets the SELECTED writable attribute.
    *
    * @return    The current value of the SELECTED attribute.
    */
   @Override
   public logical isSelected()
   {
      return selected;
   }

   /**
    * Sets the SELECTED writable attribute.
    *
    * @param    sel
    *           The new value for the SELECTED attribute.
    */
   @Override
   public void setSelected(logical sel)
   {
      selected = sel;
   }

   /**
    * Sets the SELECTED writable attribute.
    *
    * @param    sel
    *           The new value for the SELECTED attribute.
    */
   @Override
   public void setSelected(boolean sel)
   {
      selected = new logical(sel);
   }
   
   /**
    * Gets the TITLE writable attribute.
    *
    * @return   The current value of the TITLE attribute.
    */
   @Override
   public character getTitle()
   {
      return new character(base.getTitle());
   }

   /**
    * Sets the TITLE writable attribute.
    *
    * @param    title
    *           The new value for the TITLE attribute.
    */
   @Override
   public void setTitle(character title)
   {
      setTitle(title.isUnknown() ? "" : title.toStringMessage());
   }

   /**
    * Sets the TITLE writable attribute.
    *
    * @param    title
    *           The new value for the TITLE attribute.
    */
   @Override
   public void setTitle(String title)
   {
      base.setTitle(title);
      pushScreenDefinition();
   }

   /**
    * Gets the X writable attribute.
    *
    * @return   The current value of the X attribute.
    */
   @Override
   public integer getX()
   {
      return new integer(base.getX());
   }

   /**
    * Sets the X writable attribute.
    *
    * @param    x
    *           The new value for the X attribute.
    */
   public void setX(NumberType x)
   { 
      setX(x.intValue());
   }

   /**
    * Sets the X writable attribute.
    *
    * @param    x
    *           The new value for the X attribute.
    */
   public void setX(int x)
   { 
      base.setX(x);
      pushScreenDefinition();
   }

   /**
    * Gets the Y writable attribute.
    *
    * @return    The current value of the Y attribute.
    */
   @Override
   public integer getY()
   {
      return new integer(base.getY());
   }

   /**
    * Sets the Y writable attribute.
    *
    * @param    y
    *           The new value for the Y attribute.
    */
   public void setY(NumberType y)
   { 
      setY(y.intValue());
   }

   /**
    * Sets the Y writable attribute.
    *
    * @param    y
    *           The new value for the Y attribute.
    */
   public void setY(int y)
   { 
      base.setY(y);
      pushScreenDefinition();
   }

   /**
    * Set height.
    * 
    * @param   decimal
    *          Frame height.
    */
   public void setHeight(NumberType decimal)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set width.
    * 
    * @param   integer
    *          Frame width.
    */
   public void setWidth(NumberType integer)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get height.
    * 
    * @return  widget height. 
    */
   @Override
   public decimal getHeight()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get width.
    * 
    * @return  widget width. 
    */
   @Override
   public decimal getWidth()
   {
      throw new UnsupportedOperationException("Not implemented");
   }
}
