/*
** Module   : FieldGroup.java
** Abstract : groups widgets in the frame on the server side
**
** Copyright (c) 2008-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 GES 20080318   @37589 Created initial version to group widgets in
**                           a frame on the server.
** 002 NVS 20081211   @40873 getParent() method is overriden here to get
**                           the frame or another field group as the
**                           parent.
** 003 GES 20081212   @40883 Only BaseEntity can accept a field group (at
**                           this time), so safety code was added.
** 004 GES 20081230   @41002 Return a NullWidget instance instead of null
**                           for safety.
** 005 SIY 20101122          Cleaned up imports.
** 006 SIY 20111101          Updated imports.
** 007 CA  20130221          Refactored usage of TYPE attribute.
** 008 CA  20130323          The PARENT attribute must be a CommonWidget, not Object.
** 019 CA  20130927          Added LegacyResource annotation, to determine resource type.
**                           Removed NullWidget usage and fixed getFirst/LastChild to return a 
**                           handle instance.
** 020 CA  20131115          Changes related to properly instantiate dynamic resources.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

import java.util.List;
 
/**
 * Groups widgets together into a list of siblings that can be accessed
 * via the first or last entry in the list.
 */
@LegacyResource(resource = LegacyResource.FIELD_GROUP)
public class FieldGroup
extends BaseEntity
implements Group
{
   /** The list of contained widgets. */
   private List<GenericWidget> widgets = null;
   
   /**
    * Creates an instance to groups related widgets in a frame.
    *
    * @param    widgets
    *           The widgets contained in this instance. This is used as a
    *           read-only resource.
    */
   public FieldGroup(List<GenericWidget> widgets, boolean asLast)
   {
      super(false, new BaseConfig(ComponentConfig.FIELD_GROUP, null), asLast);
      this.widgets = widgets;
      
      if (widgets != null)
      {
         for (GenericWidget next : widgets)
         {
            if (next instanceof BaseEntity)
            {
               BaseEntity be = (BaseEntity) next;
               be.setGroup(this);
            }
         }
      }
   }

   /**
    * Gets the PARENT writable attribute.
    * This is an override for the BaseEntity's implementation.
    * If this field group is a member of another field group,
    * that field group is returned as parent.
    * Otherwise, the owning frame is returned as the parent.
    *
    * @return   the parent widget for this widget
    */
   public CommonWidget getParent()
   {
      // explicitly set field group
      if (group != null)
      {
         return group;
      }

      // normal case
      return frame.asWidget();
   }

   /**
    * Accesses the first widget in the list of contained widgets.
    *
    * @return   The first widget or an unknown handle if no widgets are contained.
    */
   public handle getFirstChild()
   {
      return new handle((widgets == null) ? null : widgets.get(0));
   }

   /**
    * Accesses the last widget in the list of contained widgets.
    *
    * @return   The last widget or an unknown handle if no widgets are contained.
    */
   public handle getLastChild()
   {
      return new handle((widgets == null) ? null : widgets.get(widgets.size() - 1));
   }
   
   /**
    * Accesses the next widget in the list of contained widgets.
    *
    * @param    child
    *           The widget in the list for which to find the following child.
    *
    * @return   The next widget or a <code>null</code> if there is no subsequent widget.
    */
   public GenericWidget getNextChild(GenericWidget child)
   {
      if (widgets == null || child == null)
         return null;
      
      int oldIdx = widgets.indexOf(child);
      int newIdx = oldIdx + 1;
      
      return (oldIdx < 0 || newIdx >= widgets.size()) ? null : widgets.get(newIdx);
   }

   /**
    * Accesses the previous widget in the list of contained widgets.
    *
    * @param    child
    *           The widget in the list for which to find the preceding child.
    *
    * @return   The previous widget or <code>null</code> if there is no preceding widget.
    */
   public GenericWidget getPrevChild(GenericWidget child)
   {
      if (widgets == null || child == null)
         return null;
      
      int oldIdx = widgets.indexOf(child);
      int newIdx = oldIdx - 1;
      
      return (oldIdx < 0 || newIdx < 0) ? null : widgets.get(newIdx);
   }
}
